#include <Servo.h>

#define trigPin 7
#define echoPin 6
#define led1 13
#define led2 10
#define buzzer 2

const int forwardPin1 = 8;
const int backwardPin1 = 12;
const int forwardPin2 = 9;
const int backwardPin2 = 11;
const int servoPin1 = 5; // Define the first servo pin
const int servoPin2 = 3; // Define the second servo pin

int sound = 2500;
unsigned long previousMillis = 0;
const long interval = 200; // Interval for blinking (200 ms for faster flashing)
bool ledState = LOW;

const int thresholdDistance = 40; // Distance threshold in cm

Servo servoMotor1; // Create a servo object to control the first servo motor
Servo servoMotor2; // Create a servo object to control the second servo motor

void setup() {
  Serial.begin(9600);
  
  // Set Motor pins as outputs
  pinMode(forwardPin1, OUTPUT);
  pinMode(backwardPin1, OUTPUT);
  pinMode(forwardPin2, OUTPUT);
  pinMode(backwardPin2, OUTPUT);
  
  // Set Ultrasonic Sensor pins
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  // Set LED and Buzzer pins
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  pinMode(buzzer, OUTPUT);

  // Turn on LEDs initially
  digitalWrite(led1, HIGH); 
  digitalWrite(led2, HIGH); 

  // Attach the servos to the pins and set their initial positions
  servoMotor1.attach(servoPin1);
  servoMotor1.write(180); // Set servo1 to initial position
  servoMotor2.attach(servoPin2);
  servoMotor2.write(0); // Set servo2 to initial position
}

void loop() {
    // Check distance
  int distance = getDistance();
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.println(" cm");
  
  // If an object is detected within the threshold distance, take actions
  if (distance <= thresholdDistance && distance > 0) {
    stopMotors();
    flashLEDsAndSoundBuzzer();
    delay(200); // Small delay before turning
    turnRandomly();
    delay(200); // Small delay before moving straight again
  }
  // Move straight and swing servos
  moveStraightWithSwinging();

  //delay(1); // Reduced delay between measurements
}

void moveStraightWithSwinging() {
  digitalWrite(forwardPin1, HIGH);
  digitalWrite(backwardPin1, LOW);
  digitalWrite(forwardPin2, HIGH);
  digitalWrite(backwardPin2, LOW);

  // Swing servos back and forth
  for (int pos = 0; pos <= 90; pos += 1) { // Move servos from 0 to 90 degrees
    servoMotor1.write(180 - pos); // Move servoMotor1 counterclockwise
    servoMotor2.write(pos); // Move servoMotor2 clockwise
    delay(2); // Adjust delay for desired swinging speed
  }
  for (int pos = 90; pos >= 0; pos -= 1) { // Move servos from 90 to 0 degrees
    servoMotor1.write(180 - pos); // Move servoMotor1 clockwise
    servoMotor2.write(pos); // Move servoMotor2 counterclockwise
    delay(2); // Adjust delay for desired swinging speed
  }
}

void stopMotors() {
  digitalWrite(forwardPin1, LOW);
  digitalWrite(backwardPin1, LOW);
  digitalWrite(forwardPin2, LOW);
  digitalWrite(backwardPin2, LOW);
}

void turnRandomly() {
  int turnDirection = random(2); // Generate a random number: 0 or 1
  if (turnDirection == 0) {
    // Turn left
    digitalWrite(forwardPin1, LOW);
    digitalWrite(backwardPin1, HIGH);
    digitalWrite(forwardPin2, HIGH);
    digitalWrite(backwardPin2, LOW);
  } else {
    // Turn right
    digitalWrite(forwardPin1, HIGH);
    digitalWrite(backwardPin1, LOW);
    digitalWrite(forwardPin2, LOW);
    digitalWrite(backwardPin2, HIGH);
  }
  delay(500); // Delay for the turn (adjust as needed)
  stopMotors(); // Stop after the turn
}

void flashLEDsAndSoundBuzzer() {
  unsigned long currentMillis = millis();
  unsigned long buzzerDuration = 1000; // Duration of buzzer sound (in milliseconds)
  while (millis() - currentMillis < buzzerDuration) { // Flash LEDs, sound buzzer, and move servos for the specified duration
    if (millis() - previousMillis >= interval) { // Check if it's time to toggle the LED state
      previousMillis = millis();
      ledState = !ledState; // Toggle the LED state
    }
    digitalWrite(led1, ledState);
    digitalWrite(led2, ledState);
    tone(buzzer, sound);

    // Move servo motors 90 degrees
    servoMotor1.write(90); // Rotate servoMotor1 to 90 degrees counterclockwise (upwards)
    servoMotor2.write(90); // Rotate servoMotor2 to 90 degrees clockwise (upwards)
  }
  digitalWrite(led1, HIGH);
  digitalWrite(led2, HIGH);
  noTone(buzzer);

  // Reset servo motors to initial position (0 degrees)
  servoMotor1.write(180);
  servoMotor2.write(0);

  // Stop spinning after the specified duration
  stopMotors();
}

int getDistance() {
  // Trigger the ultrasonic pulse
  digitalWrite(trigPin, LOW); 
  delayMicroseconds(1); 
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(5); 
  digitalWrite(trigPin, LOW);

  // Measure the duration of the echo
  long duration = pulseIn(echoPin, HIGH);

  // Calculate the distance in centimeters
  int distance = (duration / 2) / 29.1;
  return distance;
}